@@ -17,11 +17,6 @@
 #include <X11/StringDefs.h>
 #endif
 
-#ifdef OPENGL
-#include <GL/gl.h>
-#include <GL/glx.h>
-#endif /* OPENGL */
-
 #ifdef TCL_WRAPPER
 #include <tk.h>
 #endif
@@ -579,6 +574,7 @@ xcImage *xcImageCreate(int width, int height)
    xcImage *img = XCreateImage(dpy, DefaultVisual(dpy, screen),
 	 DefaultDepth(dpy, screen), ZPixmap, 0, NULL, width, height, 8, 0);
    img->data = (char *) malloc(height * img->bytes_per_line);
+   return img;
 }
 
 void xcImageDestroy(xcImage *img)
