@@ -37,7 +37,6 @@ extern XCWindowData *areawin;
 extern Globaldata xobjs;
 extern short fontcount;
 extern fontinfo *fonts;
-extern int *appcolors;
 extern colorindex *colorlist;
 extern char _STR[150];
 
@@ -60,6 +59,9 @@ short del = 64;
 
 char *evaluate_expr(objectptr thisobj, oparamptr ops, objinstptr pinst)
 {
+   UNUSED(thisobj);
+   UNUSED(pinst);
+
    if (ops->type != XC_EXPR) return NULL;
    return strdup(ops->parameter.expr);
 }
@@ -98,7 +100,6 @@ void joinlabels()
       return;
    }
 
-   XcSetFunction(GXcopy);
    SetForeground(dpy, areawin->gc, BACKGROUND);
 
    for (jl = areawin->selectlist; jl < areawin->selectlist +
@@ -573,7 +574,7 @@ void charprint(char *sout, stringpart *strptr, int locpos)
    switch (strptr->type) {
       case TEXT_STRING:
 	 if (strptr->data.string) {
-            if (locpos > strlen(strptr->data.string)) {
+            if (locpos > (int) strlen(strptr->data.string)) {
 	       strcpy(sout, "<ERROR>");
 	    }
             else sc = *(strptr->data.string + locpos);
@@ -616,7 +617,7 @@ void charprinttex(char *sout, stringpart *strptr, int locpos)
    switch (strptr->type) {
       case TEXT_STRING:
 	 if (strptr->data.string) {
-            if (locpos > strlen(strptr->data.string)) {
+            if (locpos > (int) strlen(strptr->data.string)) {
 	       strcpy(sout, "<ERROR>");
 	    }
             else sc = *(strptr->data.string + locpos);
@@ -629,7 +630,7 @@ void charprinttex(char *sout, stringpart *strptr, int locpos)
 	    *sout = '\0';
 	 break;
       case FONT_NAME:
-	 sprintf(sout, "");
+	 *sout = '\0';
 	 break;
       case FONT_SCALE:
       case KERN:
@@ -769,6 +770,7 @@ char *textprintnet(char *prefix, char *pinstring, Genericlist *sublist)
    char *newstr, *sptr;
    buslist *sbus;
    int i;
+   UNUSED(pinstring);
 
    if (sublist->subnets == 0) {
       newstr = (char *)malloc(strlen(prefix) + 10); 
@@ -808,7 +810,8 @@ int textcompx(stringpart *string, char *text, Boolean exact, objinstptr localins
    stringpart *strptr;
    char *tptr = text;
    char *sptr;
-   int rval, llen = strlen(text), slen;
+   int rval;
+   size_t llen = strlen(text), slen;
    Boolean has_text = FALSE;
 
    for (strptr = string; strptr != NULL; strptr = nextstringpart(strptr, localinst)) {
@@ -1011,7 +1014,7 @@ Genericlist *break_up_bus(labelptr blab, objinstptr thisinst, Genericlist *netli
    static Genericlist *subnets = NULL;
    char *tptr;
    buslist *sbus, *jbus;
-   int bpos, istart, iend, tlen, i, j, netstart, matched;
+   int istart, iend, i, j, netstart, matched;
    char *buspos, *busend, *busptr;
 
    if (pin_is_bus(blab, thisinst) == FALSE) return NULL;
@@ -1023,7 +1026,6 @@ Genericlist *break_up_bus(labelptr blab, objinstptr thisinst, Genericlist *netli
    subnets->subnets = 0;
 
    tptr = textprint(blab->string, thisinst);
-   tlen = strlen(tptr) + 1;
    buspos = strchr(tptr, areawin->buschar);
 
    /* The notation "(...)" with NO TEXT preceding the opening bus	*/
@@ -1038,7 +1040,6 @@ Genericlist *break_up_bus(labelptr blab, objinstptr thisinst, Genericlist *netli
 
    netstart = (netlist->subnets == 0) ? netlist->net.id : 0;
 
-   bpos = (int)(buspos - tptr);
    busend = find_delimiter(buspos);
 
    if (busend == NULL) {
@@ -1163,7 +1164,7 @@ int stringcomprelaxed(stringpart *string1, stringpart *string2,
 		  buspos = strchr(strptr1->data.string, matchchar);
 		  if (buspos != NULL) {
 		     bpos = (int)(buspos - (char *)(strptr1->data.string));
-		     if (strlen(strptr2->data.string) > bpos) {
+		     if ((int) strlen(strptr2->data.string) > bpos) {
 		        if (!strcmp(strptr1->data.string + bpos,
 			  	strptr2->data.string + bpos)) {
 		           in_bus = 2;
@@ -1413,7 +1414,7 @@ stringpart *stringcopyback(stringpart *string, objinstptr thisinst)
 /*---------------------------------------------------------------------*/
 
 #ifndef HAVE_CAIRO
-float UDrawChar(u_char code, short styles, short ffont, int groupheight,
+static float UDrawChar(u_char code, short styles, short ffont, int groupheight,
 	int passcolor, float passwidth)
 {
    objectptr drawchar;
@@ -1438,7 +1439,8 @@ float UDrawChar(u_char code, short styles, short ffont, int groupheight,
    drawchar = fonts[ffont].encoding[(u_char)code];
    charinst.thisobject = drawchar;
 
-   localwidth = (drawchar->bbox.lowerleft.x + drawchar->bbox.width) * fonts[ffont].scale;
+   localwidth = (drawchar->bbox.lowerleft.x + drawchar->bbox.width)
+	 * fonts[ffont].scale;
 
    if ((fonts[ffont].flags & 0x22) == 0x22) { /* font is derived and italic */
       USlantCTM(DCTM, 0.25);  		/* premultiply by slanting function */
@@ -1460,7 +1462,33 @@ float UDrawChar(u_char code, short styles, short ffont, int groupheight,
    }
    return localwidth;
 }
-#endif /* HAVE_CAIRO */
+#endif /* !HAVE_CAIRO */
+
+/*---------------------------------------------------------------------*/
+/* Draw a string at position offset, starting at the start index and   */
+/* ending before the end index. The offset is updated on return.       */
+/*---------------------------------------------------------------------*/
+
+#ifndef HAVE_CAIRO
+void UDrawCharString(u_char *text, int start, int end, XfPoint *offset,
+      short styles, short ffont, int groupheight, int passcolor, float tmpscale)
+{
+   int idx;
+   
+   float tmpthick = ((fonts[ffont].flags & 0x21) == 0x21) ?  4.0 : 2.0;
+   for (idx = start; idx < end; idx++)
+   {
+      XPoint p;
+      p.x = offset->x;
+      p.y = offset->y;
+      UPushCTM();
+      UPreMultCTM(DCTM, p, tmpscale, 0);
+      offset->x += UDrawChar(text[idx], styles, ffont, groupheight, passcolor,
+	    tmpthick) * tmpscale;
+      UPopCTM();
+   }
+}
+#endif /* !HAVE_CAIRO */
 
 /*----------------------------------------------------------------------*/
 /* Draw an entire string, including parameter substitutions		*/
@@ -1471,21 +1499,28 @@ void UDrawString0(labelptr drawlabel, int passcolor, objinstptr localinst,
 	Boolean drawX)
 {
    stringpart *strptr;
-   char *textptr;
+   u_char *textptr;
    short  fstyle, ffont, tmpjust;
    float  baseline;
    int    pos, group = 0;
    int    defaultcolor, curcolor, scolor;
    float  oldx;
    short  oldfont, oldstyle;
-   float  tmpscale = 1.0, natscale = 1.0, tmpthick = 2.0;
+   float  tmpscale = 1.0, natscale = 1.0;
    XfPoint newpoint;
    XPoint  bboxin[2], bboxout[2];
    u_char xm, ym;
    TextExtents tmpext;
    short *tabstops = NULL;
    short tabno, numtabs = 0;
-   int   marginstop = 0;
+#ifdef HAVE_CAIRO
+   cairo_matrix_t fm = {40., 0., 0., -40., 0., 0.}; /* TODO: Why 40? */
+
+   if (!areawin->redraw_ongoing) {
+      areawin->redraw_needed = True;
+      return;
+   }
+#endif /* HAVE_CAIRO */
 
    if (fontcount == 0) return;
 
@@ -1529,7 +1564,9 @@ void UDrawString0(labelptr drawlabel, int passcolor, objinstptr localinst,
    /* them from the circuit point to which they attach.  */
 
    if (drawlabel->pin) {
-      XPoint p = {.x = newpoint.x, .y = newpoint.y};
+      XPoint p;
+      p.x = newpoint.x;
+      p.y = newpoint.y;
       pinadjust(tmpjust, &p.x, &p.y, 1);
       newpoint.x = p.x;
       newpoint.y = p.y;
@@ -1542,7 +1579,7 @@ void UDrawString0(labelptr drawlabel, int passcolor, objinstptr localinst,
 
    bboxin[0].x = newpoint.x;
    bboxin[0].y = newpoint.y + tmpext.descent;
-   bboxin[1].x = newpoint.x + tmpext.width;
+   bboxin[1].x = newpoint.x + tmpext.maxwidth;
    bboxin[1].y = newpoint.y + tmpext.ascent;
    UTransformbyCTM(DCTM, bboxin, bboxout, 2);
    xm = (bboxout[0].x < bboxout[1].x) ? 0 : 1;
@@ -1573,16 +1610,16 @@ void UDrawString0(labelptr drawlabel, int passcolor, objinstptr localinst,
 	              oldstyle = fstyle;
 	           }
 		}
-		
-		/* simple boldface technique for derived fonts */
-
-		tmpthick = ((fonts[ffont].flags & 0x21) == 0x21) ?  4.0 : 2.0;
+#ifdef HAVE_CAIRO
+		cairo_set_font_face(areawin->cr, fonts[ffont].font_face);
+		cairo_set_font_matrix(areawin->cr, &fm);
+#endif /* HAVE_CAIRO */
 		break;
 
 	     case FONT_SCALE:
 		tmpscale = natscale * strptr->data.scale;
-	        // if (baseline == newpoint.y) /* reset top-level scale */
-		//    natscale = tmpscale;
+	        /* if (baseline == newpoint.y) */ /* reset top-level scale */
+		/*    natscale = tmpscale; */
 		break;
 
 	     case KERN:
@@ -1592,8 +1629,12 @@ void UDrawString0(labelptr drawlabel, int passcolor, objinstptr localinst,
 		
 	     case FONT_COLOR:
 		if (defaultcolor != DOFORALL) {
-		   if (strptr->data.color != DEFAULTCOLOR)
-		      curcolor = colorlist[strptr->data.color].color.pixel;
+		   if (strptr->data.color != DEFAULTCOLOR) {
+		      curcolor = strptr->data.color;
+#ifdef HAVE_CAIRO
+		      XTopSetForeground(curcolor);
+#endif
+		   }
 		   else {
 		      if (curcolor != DEFAULTCOLOR) {
 			 XTopSetForeground(defaultcolor);
@@ -1652,6 +1693,10 @@ void UDrawString0(labelptr drawlabel, int passcolor, objinstptr localinst,
 	        ffont = oldfont;	/* revert to top-level font and style */
 	        fstyle = oldstyle;
 	        newpoint.y = baseline;
+#ifdef HAVE_CAIRO
+		cairo_set_font_face(areawin->cr, fonts[oldfont].font_face);
+		cairo_set_font_matrix(areawin->cr, &fm);
+#endif /* HAVE_CAIRO */
 		break;
 
 	     case UNDERLINE:
@@ -1660,15 +1705,19 @@ void UDrawString0(labelptr drawlabel, int passcolor, objinstptr localinst,
 
 	     case OVERLINE:
 		if (strptr->nextpart != NULL && strptr->nextpart->type == TEXT_STRING) {
-		   objectptr charptr;
 		   int tmpheight;
 
 		   group = 0;
 		   for (textptr = strptr->nextpart->data.string;
 				textptr && *textptr != '\0'; textptr++) {
+#ifdef HAVE_CAIRO
+		      tmpheight = fonts[ffont].glyph_top[*textptr];
+#else /* HAVE_CAIRO */
+		      objectptr charptr;
 		      charptr = fonts[ffont].encoding[*(u_char *)textptr];
 		      tmpheight = (int)((float)charptr->bbox.height
 				* fonts[ffont].scale);
+#endif /* HAVE_CAIRO */
 		      if (group < tmpheight) group = tmpheight;
 		   }
 	           fstyle |= 16;
@@ -1678,16 +1727,28 @@ void UDrawString0(labelptr drawlabel, int passcolor, objinstptr localinst,
 	     case NOLINE:
 		break;
 
+#ifdef HAVE_CAIRO
+             case HALFSPACE:
+                newpoint.x += fonts[ffont].glyph_advance[' '] * tmpscale / 2.;
+                break;
+
+             case QTRSPACE:
+                newpoint.x += fonts[ffont].glyph_advance[' '] * tmpscale / 4.;
+                break;
+#else /* HAVE_CAIRO */
 	     case HALFSPACE: case QTRSPACE: {
-		XPoint p = {.x = newpoint.x, .y = newpoint.y};
+		XPoint p;
 		short addx;
+	       	p.x = newpoint.x;
+		p.y = newpoint.y;
 		UPushCTM();
 		UPreMultCTM(DCTM, p, tmpscale, 0);
 		addx = UDrawChar((u_char)32, fstyle, ffont, group,
-			curcolor, tmpthick);
+			curcolor, 2.);
 		newpoint.x += addx >> ((strptr->type == HALFSPACE) ? 1 : 2);
 		UPopCTM();
 		} break;
+#endif /* HAVE_CAIRO */
 
 	     case TEXT_STRING:
 		textptr = strptr->data.string;
@@ -1705,29 +1766,35 @@ void UDrawString0(labelptr drawlabel, int passcolor, objinstptr localinst,
 		   }
 		}
 
-		for (; textptr && *textptr != '\0'; textptr++) {
-		   XPoint p = {.x = newpoint.x, .y = newpoint.y};
-		   pos++;
-		   UPushCTM();
-		   UPreMultCTM(DCTM, p, tmpscale, 0);
-
-	           /* Special case of selection:  only substring is	*/
-		   /* drawn in the selection color.			*/
-
-		   scolor = curcolor;
-	           if (passcolor == DOSUBSTRING) {
-	              if (pos <= areawin->textpos && pos > areawin->textend)
-		         scolor = SELECTCOLOR;
-		      else if (pos > areawin->textpos) {
-      		         XTopSetForeground(curcolor);
-		      }
-	           }
-	           newpoint.x += UDrawChar(*textptr, fstyle, ffont,
-			group, scolor, tmpthick) * tmpscale;
-
-		   UPopCTM();
+	        scolor = curcolor;
+	        if (passcolor == DOSUBSTRING) {
+		   int len = strlen(textptr);
+		   int begin_sel = min(max(0, areawin->textend - pos), len);
+		   int end_sel = min(max(0, areawin->textpos - pos), len);
+		   if (begin_sel > 0) {
+		      UDrawCharString(textptr, 0, begin_sel, &newpoint, fstyle,
+			    ffont, group, scolor, tmpscale);
+		   }
+		   if (begin_sel < end_sel) {
+		      scolor = SELECTCOLOR;
+#ifdef HAVE_CAIRO
+      		      XTopSetForeground(scolor);
+#endif /* HAVE_CAIRO */
+		      UDrawCharString(textptr, begin_sel, end_sel, &newpoint,
+			    fstyle, ffont, group, scolor, tmpscale);
+		   }
+		   if (end_sel < len) {
+		      scolor = curcolor;
+      		      XTopSetForeground(curcolor);
+		      UDrawCharString(textptr, end_sel, len, &newpoint, fstyle,
+			    ffont, group, scolor, tmpscale);
+		   }
 		}
-		pos--;
+		else {
+	           UDrawCharString(textptr, 0, strlen(textptr),
+		         &newpoint, fstyle, ffont, group, scolor, tmpscale);
+		}
+	        pos += strlen(textptr) - 1;
 		break;
 	  }
 	  pos++;
@@ -1744,7 +1811,8 @@ void UDrawString0(labelptr drawlabel, int passcolor, objinstptr localinst,
 
    if (drawX && drawlabel->pin) UDrawXDown(drawlabel);
 
-   if ((defaultcolor != DOFORALL) && (passcolor != curcolor)) {
+   if ((defaultcolor != DOFORALL) && (passcolor != curcolor) &&
+		(passcolor != DOSUBSTRING)) {
       XTopSetForeground(passcolor);
    }
 }
@@ -1778,13 +1846,12 @@ TextExtents ULength(labelptr drawlabel, objinstptr localinst,
    float lasttotal = xtotal;
    stringpart *strptr;
    u_char *textptr;
-   objectptr *somebet = NULL, chptr;
+   objectptr *somebet = NULL;
    short locpos = 0, lastpos = 0;
    float ykern = 0.0;
    TextExtents retext;
    short *tabstops = NULL;
    short tabno, numtabs = 0;
-   int marginstop = 0;
    Boolean dobreak = FALSE;
 #ifdef HAVE_CAIRO
    fontinfo *font = NULL;
@@ -1823,33 +1890,30 @@ TextExtents ULength(labelptr drawlabel, objinstptr localinst,
 	    natscale = strscale = oldscale;
 	    ykern = 0.0;
 	    retext.base -= BASELINE;
-	    retext.maxwidth = max(retext.width, xtotal);
+	    retext.maxwidth = max(retext.width, retext.maxwidth);
+	    retext.maxwidth = max(retext.maxwidth, xtotal);
 	    xtotal = 0.5;
 	    break;
 	 case HALFSPACE: 
-#ifdef HAVE_CAIRO
-	    if (font && font->font_face)
-	       xtotal += xc_cairo_get_char_extents(font, ' ', NULL, NULL)
-		     * locscale * natscale / 2.;
-	    else
-#endif /* HAVE_CAIRO */
 	    if (somebet) {
-	       chptr = (*(somebet + 32));
+#ifdef HAVE_CAIRO
+	       xtotal += font->glyph_advance[' '] * locscale * strscale / 2.;
+#else /* HAVE_CAIRO */
+	       objectptr chptr = (*(somebet + 32));
 	       xtotal += (float)(chptr->bbox.width + chptr->bbox.lowerleft.x)
 			* locscale * natscale / 2.;
+#endif /* HAVE_CAIRO */
 	    }
 	    break;
 	 case QTRSPACE:
-#ifdef HAVE_CAIRO
-	    if (font && fonts->font_face)
-	       xtotal += xc_cairo_get_char_extents(font, ' ', NULL, NULL)
-		     * locscale * natscale / 4.;
-	    else
-#endif /* HAVE_CAIRO */
 	    if (somebet) {
-	       chptr = (*(somebet + 32));
-	       xtotal += (float)(chptr->bbox.width + chptr->bbox.lowerleft.x) 
+#ifdef HAVE_CAIRO
+	       xtotal += font->glyph_advance[' '] * locscale * strscale / 4.;
+#else /* HAVE_CAIRO */
+	       objectptr chptr = (*(somebet + 32));
+	       xtotal += (float)(chptr->bbox.width + chptr->bbox.lowerleft.x)
 			* locscale * natscale / 4.;
+#endif /* HAVE_CAIRO */
 	    }
 	    break;
 	 case TABBACKWARD:	/* find first tab value with x < xtotal */
@@ -1876,8 +1940,8 @@ TextExtents ULength(labelptr drawlabel, objinstptr localinst,
 	    break;
 	 case FONT_SCALE:
 	    strscale = natscale * strptr->data.scale;
-	    // if (ykern == 0.0)
-	    //   natscale = strscale;
+	    /* if (ykern == 0.0) */
+	    /*   natscale = strscale; */
 	    break;
 	 case KERN:
 	    xtotal += strptr->data.kern[0];
@@ -1918,17 +1982,17 @@ TextExtents ULength(labelptr drawlabel, objinstptr localinst,
 	       locpos++;
 
 #ifdef HAVE_CAIRO
-	       if (font->font_face)
-		  advance = xc_cairo_get_char_extents(font, *textptr, &top,
-			&bottom);
-               else
-#endif /* HAVE_CAIRO */
+	       advance = font->glyph_advance[*textptr];
+	       top = font->glyph_top[*textptr];
+	       bottom = font->glyph_bottom[*textptr];
+#else /* HAVE_CAIRO */
 	       {
-	          chptr = *(somebet + *textptr);
+	       	  objectptr chptr = *(somebet + *textptr);
 	          advance = chptr->bbox.width + chptr->bbox.lowerleft.x;
 	          top = chptr->bbox.height + chptr->bbox.lowerleft.y;
 	          bottom = chptr->bbox.lowerleft.y;
-               }
+	       }
+#endif /* HAVE_CAIRO */
 	       xtotal += advance * locscale * strscale;
 	       retext.ascent = max(retext.ascent, (short)(retext.base + ykern
 		     + top * locscale * strscale));
@@ -1973,7 +2037,7 @@ TextExtents ULength(labelptr drawlabel, objinstptr localinst,
 
 void RemoveMarginNewlines(labelptr settext, objinstptr localinst)
 {
-   stringpart *strptr, *nextptr;
+   stringpart *strptr;
    int strpos = 0;
 
    for (strptr = settext->string; strptr != NULL;
@@ -1981,7 +2045,7 @@ void RemoveMarginNewlines(labelptr settext, objinstptr localinst)
       switch (strptr->type) {
 	 case RETURN:
 	    if (strptr->data.flags != 0) {
-	       // Remove (without merge)
+	       /* Remove (without merge) */
 	       strptr = deletestring0(strptr, &settext->string, localinst, FALSE);
 	       if (strpos <= areawin->textpos) areawin->textpos--;
 	    }
@@ -2013,7 +2077,7 @@ void InsertMarginNewlines(labelptr settext, objinstptr localinst)
 {
    stringpart *strptr, *lastseg = NULL;
    int margin = 0;
-   int strpos = 0, locpos, tmplen, slen, savelen;
+   int strpos = 0, locpos, slen, savelen;
    TextExtents tmpext;
 
    /* 1) Find the position of the margin stop.  Track position	*/
@@ -2039,7 +2103,7 @@ void InsertMarginNewlines(labelptr settext, objinstptr localinst)
       }
       if (margin > 0) break;
    }
-   if (margin == 0) return;	// Should not happen. . .
+   if (margin == 0) return;	/* Should not happen. . . */
    lastseg = strptr;
 
    /* 2) Compute the drawn string length at each word break.  When a	*/
@@ -2067,15 +2131,15 @@ void InsertMarginNewlines(labelptr settext, objinstptr localinst)
 	    /* it exceeds the margin by itself.					*/
 	    if (savelen > slen) slen = savelen;
 	    if (slen > 0) {
-	       // Split string at word separation before the margin.
+	       /* Split string at word separation before the margin. */
 
 	       while ((slen > 0) && (*(strptr->data.string + slen) == ' ')) slen++;
 	       strptr = splitstring(strpos + slen, &settext->string, localinst);
 	       strptr = nextstringpart(strptr, localinst);
 	    }
 
-	    // Insert a carriage return, if the previous segment was not
-	    // already one.
+	    /* Insert a carriage return, if the previous segment was not */
+	    /* already one. */
 	    if (slen > 0 || (lastseg->type != RETURN)) {
 	       strptr = makesegment(&settext->string, strptr);
 	       strptr->type = RETURN;
@@ -2090,7 +2154,7 @@ void InsertMarginNewlines(labelptr settext, objinstptr localinst)
 	    strpos += strlen(strptr->data.string);
       }
       else if (strptr->type == MARGINSTOP) {
-	 // Allows multiple margin stops in the same text block
+	 /* Allows multiple margin stops in the same text block */
 	 margin = strptr->data.width;
 	 strpos++;
       }
@@ -2131,7 +2195,7 @@ void CheckMarginStop(labelptr settext, objinstptr localinst, Boolean force)
       }
    }
    else {
-      // In case the Margin Stop directive just got deleted. . .
+      /* In case the Margin Stop directive just got deleted. . . */
       RemoveMarginNewlines(settext, localinst);
    }
 }
@@ -2142,7 +2206,6 @@ void CheckMarginStop(labelptr settext, objinstptr localinst, Boolean force)
 
 void undrawtextsimple(labelptr settext)
 {
-   SetFunction(dpy, areawin->gc, GXcopy);
    XTopSetForeground(BACKGROUND);
    UDrawString(settext, DOFORALL, areawin->topinstance);
 }
@@ -2210,8 +2273,7 @@ void redrawtext(labelptr curlabel)
 void composefontlib(short cfont)
 {
   /* genericptr *pgen; (jdk) */
-   objinstptr *drawinst;
-   objectptr *curlib, libobj, nullobj;
+   objectptr nullobj;
    objectptr directory = xobjs.libtop[FONTLIB]->thisobject;
    short visobjects, i;
    polyptr *drawbox;
@@ -2219,10 +2281,6 @@ void composefontlib(short cfont)
 
    reset(directory, NORMAL);
 
-   /* Create a pointer to the font library */
-
-   curlib = xobjs.fontlib.library;
-
    /* Find the number of non-null characters.  Do this by assuming */
    /* that all fonts encode nullchar at position zero.		   */
 
@@ -2243,7 +2301,9 @@ void composefontlib(short cfont)
 
    for (i = 0; i < 256; i++) {
       stringpart *sfont, *schar;
-      char character[] = {i, 0};
+      labelptr label;
+      u_char character[2] = {0, 0};
+      character[0] = i;
 
       if (fonts[cfont].encoding[i] == nullobj)
 	 continue;
@@ -2256,8 +2316,8 @@ void composefontlib(short cfont)
       schar->type = TEXT_STRING;
       schar->data.string = strdup(character);
       schar->nextpart = NULL;
-      labelptr label = new_label(xobjs.libtop[FONTLIB], sfont, NORMAL,
-		      (i % 16) * del + del / 2, -(i / 16) * del - del / 2 - 32 /2);
+      label = new_label(xobjs.libtop[FONTLIB], sfont, NORMAL,
+	    (i % 16) * del + del / 2, -(i / 16) * del - del / 2 - 32 /2);
       label->justify = NOTLEFT;
       label->color = DEFAULTCOLOR;
    }
