@@ -1115,7 +1115,12 @@ void undo_finish_series()
 
 void undo_action()
 {
-   short idx = undo_one_action();
+   short idx;
+
+   // Cannot undo while in the middle of an undo series.  Failsafe.
+   if (undo_collect != (u_char)0) return;
+
+   idx = undo_one_action();
    while (xobjs.undostack && xobjs.undostack->idx == idx)
       undo_one_action();
 }
@@ -1511,7 +1516,12 @@ short redo_one_action()
 
 void redo_action()
 {
-   short idx = redo_one_action();
+   short idx;
+
+   // Cannot redo while in the middle of an undo series.  Failsafe.
+   if (undo_collect != (u_char)0) return;
+
+   idx = redo_one_action();
    while (xobjs.redostack && xobjs.redostack->idx == idx)
       redo_one_action();
 }
